const std = @import("std");
const Version = enum {
    raylib51,
    raylib55,
};
pub fn build(b: *std.Build) void {
    const target = b.standardTargetOptions(.{});
    const optimize = b.standardOptimizeOption(.{});

    const wf = b.addWriteFiles();

    const opt_version = b.option(Version, "version", "Raylib version");
    if (opt_version == null) {
        b.getInstallStep().dependOn(&b.addFail("test").step);
    }
    const version = switch (opt_version orelse .raylib51) {
        inline else => |v| @tagName(v),
    };

    const exe = b.addExecutable(.{
        .name = version,
        .root_source_file = wf.add("src/" ++ version ++ ".zig",
            \\const std = @import("std");
            \\const print = std.debug.print;
            \\const sleep = std.time.sleep;
            \\const raylib = @import("raylib");
            \\pub fn main() void {
            \\    raylib.SetTraceLogLevel(raylib.LOG_ERROR);
            \\    raylib.InitWindow(100, 100, "title");
            \\    print("Version: {s}:\n", .{
        ++ "\"" ++ version ++ "\"," ++
            \\    });
            \\    print("Monitor width: {d}\n", .{raylib.GetMonitorWidth(0)});
            \\    print("Screen width: {d}\n", .{raylib.GetScreenWidth()});
            \\    print("DPI: {d}\n", .{raylib.GetWindowScaleDPI().x});
            \\    raylib.SetTargetFPS(30);
            \\    const camera = raylib.Camera3D{
            \\        .position = .{ .x = 1 },
            \\        .up = .{ .y = 1 },
            \\        .fovy = 45,
            \\    };
            \\    while (!raylib.WindowShouldClose()) {
            \\        raylib.ClearBackground(raylib.RED);
            \\        raylib.BeginDrawing();
            \\        defer raylib.EndDrawing();
            \\        {
            \\            raylib.BeginMode3D(camera);
            \\            defer raylib.EndMode3D();
            \\            raylib.DrawGrid(10, 1);
            \\            raylib.DrawCube(.{}, 1, 1, 1, raylib.GREEN);
            \\        }
            \\    }
            \\    
            \\}
        ),
        .target = target,
        .optimize = optimize,
    });
    b.installArtifact(exe);

    const raylib = b.dependency(version, .{
        .target = target,
        .optimize = optimize,
    });
    const raylib_c = b.addTranslateC(.{
        .root_source_file = wf.add(version ++ ".h",
            \\#include <raylib.h>
            \\#include <rlgl.h>
        ),
        .target = target,
        .optimize = optimize,
    });
    raylib_c.defineCMacro("_GLFW_WAYLAND", null);
    raylib_c.addIncludePath(raylib.path("src"));

    exe.root_module.addImport("raylib", raylib_c.createModule());
    exe.root_module.linkLibrary(raylib.artifact("raylib"));

    const run_cmd = b.addRunArtifact(exe);
    run_cmd.step.dependOn(b.getInstallStep());
    const run_step = b.step("run", "Run the app");
    run_step.dependOn(&run_cmd.step);
}
